@@ -50,6 +50,7 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "genomeTransformType", &pGe.transform.typeString));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "genomeTransformVCF", &pGe.transform.vcfFile));
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "genomeTransformOutput", &pGe.transform.output));
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "genomeChrSetMitochondrial", &pGe.chrSet.mitoStrings));
 
     //read
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readFilesType", &readFilesType));
@@ -76,7 +77,7 @@ Parameters::Parameters() {//initalize parameters info
 
     //limits
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitGenomeGenerateRAM", &limitGenomeGenerateRAM));
-    parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitIObufferSize", &limitIObufferSize));
+    parArray.push_back(new ParameterInfoVector <uint64>   (-1, -1, "limitIObufferSize", &limitIObufferSize));
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitOutSAMoneReadBytes", &limitOutSAMoneReadBytes));
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitOutSJcollapsed", &limitOutSJcollapsed));
     parArray.push_back(new ParameterInfoScalar <uint>   (-1, -1, "limitOutSJoneRead", &limitOutSJoneRead));
@@ -117,8 +118,10 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <uint>     (-1, -1, "outSAMattrIHstart", &outSAMattrIHstart));
     parArray.push_back(new ParameterInfoScalar <int>        (-1, -1, "outSAMtlen", &outSAMtlen));
 
-
-   //output SJ filtering
+    //outSJ
+    parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "outSJtype", &outSJ.type));
+    
+    //output SJ filtering
     parArray.push_back(new ParameterInfoScalar <string>  (-1, -1, "outSJfilterReads", &outSJfilterReads));
     parArray.push_back(new ParameterInfoVector <int32>   (-1, -1, "outSJfilterCountUniqueMin", &outSJfilterCountUniqueMin));
     parArray.push_back(new ParameterInfoVector <int32>   (-1, -1, "outSJfilterCountTotalMin", &outSJfilterCountTotalMin));
@@ -285,10 +288,15 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloUMIposition",&pSolo.umiPositionStr));
     parArray.push_back(new ParameterInfoVector <string>   (-1, -1, "soloCellFilter",&pSolo.cellFilter.type));
     parArray.push_back(new ParameterInfoVector <string>   (-1, -1, "soloUMIfiltering",&pSolo.umiFiltering.type));
+    
+    parArray.push_back(new ParameterInfoVector <string>   (-1, -1, "soloMultiMappers", &pSolo.multiMap.typesIn));
+    
     parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloClusterCBfile",&pSolo.clusterCBfile));
     parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloOutFormatFeaturesGeneField3",&pSolo.outFormat.featuresGeneField3));
+    
     parArray.push_back(new ParameterInfoVector <string>   (-1, -1, "soloInputSAMattrBarcodeSeq",&pSolo.samAtrrBarcodeSeq));
     parArray.push_back(new ParameterInfoVector <string>   (-1, -1, "soloInputSAMattrBarcodeQual",&pSolo.samAtrrBarcodeQual));
+    parArray.push_back(new ParameterInfoScalar <string>   (-1, -1, "soloCellReadStats",&pSolo.readStats.type));
 
     parameterInputName.push_back("Default");
     parameterInputName.push_back("Command-Line-Initial");
@@ -368,6 +376,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
 
     inOut->logMain << "STAR version=" << STAR_VERSION << "\n";
     inOut->logMain << "STAR compilation time,server,dir=" << COMPILATION_TIME_PLACE << "\n";
+    inOut->logMain << "STAR git: " << GIT_BRANCH_COMMIT_DIFF << "\n";
     #ifdef COMPILE_FOR_LONG_READS
            inOut->logMain << "Compiled for LONG reads" << "\n";
     #endif
@@ -832,17 +841,6 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
                 inOut->outUnmappedReadsStream[imate].open(ff.str().c_str());
             };
         };
-
-        if (outFilterType=="Normal") {
-            outFilterBySJoutStage=0;
-        } else if (outFilterType=="BySJout") {
-            outFilterBySJoutStage=1;
-        } else {
-            ostringstream errOut;
-            errOut <<"EXITING because of FATAL input ERROR: unknown value of parameter outFilterType: " << outFilterType <<"\n";
-            errOut <<"SOLUTION: specify one of the allowed values: Normal | BySJout\n";
-            exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
-        };
     };
 
     if (outSAMmapqUnique<0 || outSAMmapqUnique>255) {
@@ -851,20 +849,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             errOut <<"SOLUTION: specify outSAMmapqUnique within the range of 0 to 255\n";
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
-
-    // in/out buffers
-    #define BUFFER_InSizeFraction 0.5
-    if (limitIObufferSize<limitOutSJcollapsed*Junction::dataSize+1000000) {
-        ostringstream errOut;
-        errOut <<"EXITING because of FATAL INPUT ERROR: --limitIObufferSize="<<limitIObufferSize <<" is too small for ";
-        errOut << "--limitOutSJcollapsed*"<<Junction::dataSize<<"="<< limitOutSJcollapsed<<"*"<<Junction::dataSize<<"="<<limitOutSJcollapsed*Junction::dataSize<<"\n";
-        errOut <<"SOLUTION: re-run STAR with larger --limitIObufferSize or smaller --limitOutSJcollapsed\n";
-        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
-    };
-    chunkInSizeBytesArray=(uint) int((limitIObufferSize-limitOutSJcollapsed*Junction::dataSize)*BUFFER_InSizeFraction)/2;
-    chunkOutBAMsizeBytes= (uint) int((1.0/BUFFER_InSizeFraction-1.0)*chunkInSizeBytesArray*2.0);
-    chunkInSizeBytes=chunkInSizeBytesArray-2*(DEF_readSeqLengthMax+1)-2*DEF_readNameLengthMax;//to prevent overflow
-
+        
     //variation
     var.yes=false;
     if (var.vcfFile!="-") {
@@ -902,8 +887,6 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     
     //quantification parameters
     quant.yes=false;
-    quant.geneFull.yes=false;
-    quant.gene.yes=false;
     quant.geCount.yes=false;
     quant.trSAM.yes=false;
     quant.trSAM.bamYes=false;
@@ -947,6 +930,10 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             };
         };
     };
+    //these may be set in STARsolo or in SAM attributes
+    quant.geneFull.yes=false;
+    quant.gene.yes=false;
+    
 
     outSAMstrandField.type=0; //none
     if (outSAMstrandField.in=="None") {
@@ -1128,7 +1115,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         alignSoftClipAtReferenceEnds.yes=false;
     } else {
         ostringstream errOut;
-        errOut << "EXITING because of fatal PARAMETERS error: unrecognized option in of --alignSoftClipAtReferenceEnds="<<alignSoftClipAtReferenceEnds.in<<"\n";
+        errOut << "EXITING because of fatal PARAMETERS error: unrecognized option in --alignSoftClipAtReferenceEnds   "<<alignSoftClipAtReferenceEnds.in<<"\n";
         errOut << "SOLUTION: use allowed option: Yes or No";
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
@@ -1138,6 +1125,64 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         outSAMreadIDnumber=true;
     };
 
+
+    //////////////////////////////////////////////////////////////////////////////////
+    ////////////////////////////////////////////////////////////////////////////////// these parameters do not depend on other parameters
+    /////////////////////////////////////////////////////////////////////////////////
+    
+    ////////////////////////////////////////////////////////// limitIObufferSize
+    /* old before 2.7.9
+    // in/out buffers
+    #define BUFFER_InSizeFraction 0.5
+    if (limitIObufferSize<limitOutSJcollapsed*Junction::dataSize+1000000) {
+        ostringstream errOut;
+        errOut <<"EXITING because of FATAL INPUT ERROR: --limitIObufferSize="<<limitIObufferSize <<" is too small for ";
+        errOut << "--limitOutSJcollapsed*"<<Junction::dataSize<<"="<< limitOutSJcollapsed<<"*"<<Junction::dataSize<<"="<<limitOutSJcollapsed*Junction::dataSize<<"\n";
+        errOut <<"SOLUTION: re-run STAR with larger --limitIObufferSize or smaller --limitOutSJcollapsed\n";
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
+    chunkInSizeBytesArray=(uint) int((limitIObufferSize-limitOutSJcollapsed*Junction::dataSize)*BUFFER_InSizeFraction)/2;
+    chunkOutBAMsizeBytes= (uint) int((1.0/BUFFER_InSizeFraction-1.0)*chunkInSizeBytesArray*2.0);
+    chunkInSizeBytes=chunkInSizeBytesArray-2*(DEF_readSeqLengthMax+1)-2*DEF_readNameLengthMax;//to prevent overflow
+    */
+    
+    if (limitIObufferSize.size() != 2) 
+        exitWithError("EXITING because of FATAL input ERROR: --limitIObufferSize requires 2 numbers since 2.7.9a.\n"
+                      "SOLUTION: specify 2 numbers in --limitIObufferSize : size of input and output buffers in bytes.\n"
+                        , std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    
+    chunkInSizeBytesArray = limitIObufferSize[0]/readNends; //array size
+    chunkInSizeBytes = chunkInSizeBytesArray-2*(DEF_readSeqLengthMax+1)-2*DEF_readNameLengthMax; //to prevent overflow - array is bigger to allow loading one read
+    chunkOutBAMsizeBytes = limitIObufferSize[1];
+    
+    
+    ///////////////////////////////////////////////////////// outSJ
+    if (outSJ.type[0] == "None") {
+        outSJ.yes = false;
+    } else if (outSJ.type[0] == "Standard") {
+        outSJ.yes = true;
+    } else {
+        exitWithError("EXITING because of FATAL input ERROR: unrecognized option in --outSJtype   " + outSJ.type[0] + '\n' +
+                      "SOLUTION: use one of the allowed options: --outSJtype   Standard    OR    None\n"
+                        , std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };
+
+    if (outFilterType=="Normal") {
+        outFilterBySJoutStage=0;
+    } else if (outFilterType=="BySJout") {
+        if (!outSJ.yes)
+            exitWithError("EXITING because of FATAL input ERROR: --outFilterType BySJout requires --outSJtype Standard\n"
+                      "SOLUTION: --outFilterType Normal    OR   --outFilterType BySJout --outSJtype Standard\n"
+                        , std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+            
+        outFilterBySJoutStage=1;
+    } else {
+        ostringstream errOut;
+        errOut <<"EXITING because of FATAL input ERROR: unknown value of parameter outFilterType: " << outFilterType <<"\n";
+        errOut <<"SOLUTION: specify one of the allowed values: Normal | BySJout\n";
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+    };    
+    
     ////////////////////////////////////////////////
     inOut->logMain << "Finished loading and checking parameters\n" <<flush;
 };
