@@ -166,7 +166,7 @@ class Parameters {
 
         int outSAMtlen;
 
-        struct {bool NH,HI,AS,NM,MD,nM,jM,jI,RG,XS,rB,vG,vA,vW,ha,ch,MC,CR,CY,UR,UY,CB,UB,GX,GN,sM,sS,sQ,cN;} outSAMattrPresent, outSAMattrPresentQuant;
+        struct {bool NH,HI,AS,NM,MD,nM,jM,jI,RG,XS,rB,vG,vA,vW,ha,ch,MC,CR,CY,UR,UY,CB,UB,GX,GN,gx,gn,sM,sS,sQ,cN;} outSAMattrPresent, outSAMattrPresentQuant;
 
         vector <int> outSAMattrOrder, outSAMattrOrderQuant;
         int outBAMcompression;
@@ -229,6 +229,11 @@ class Parameters {
         string outFilterType; //type of filtering
         int outFilterBySJoutStage; //indicates the stage of filtering by SJout
 
+        struct {
+            vector<string> type;
+            bool yes;
+        } outSJ;
+        
         //output filtering SJs
         string outSJfilterReads;
         vector <int32> outSJfilterCountUniqueMin, outSJfilterCountTotalMin;
@@ -270,7 +275,7 @@ class Parameters {
 
         //storage limits
         uint64 limitGenomeGenerateRAM;
-        uint64 limitIObufferSize; //max size of the in/out buffer, bytes
+        vector<uint64> limitIObufferSize; //max size of the in/out buffer, bytes
         uint64 limitOutSAMoneReadBytes;
         uint64 limitOutSJoneRead, limitOutSJcollapsed;
         uint64 limitBAMsortRAM;
@@ -286,42 +291,50 @@ class Parameters {
         //input
 
         struct {
-          bool yes; //if any quantification is done
+          bool yes=false; //if any quantification is done
           vector <string> mode; //quantification mode input string
 
-          struct {
-              bool yes;
-              bool bamYes;
-              bool indel;
-              bool softClip;
-              bool singleEnd;
-              int bamCompression;
-              string ban;
-          } trSAM;
+            struct {
+                bool yes=false;
+                bool bamYes;
+                bool indel;
+                bool softClip;
+                bool singleEnd;
+                int bamCompression;
+                string ban;
+            } trSAM;
 
-          struct {
-              bool yes;
-              string outFile;
-          } geCount;
+            struct {
+                bool yes=false;
+                string outFile;
+            } geCount;
 
-          struct {
-              bool yes;
-          } geneFull;
+            struct {
+                bool yes=false;
+            } geneFull;
+          
+            struct {
+                bool yes=false;
+            } geneFull_Ex50pAS;
 
-          struct {
-              bool yes;
-          } gene;
+            struct {
+                bool yes=false;
+            } geneFull_ExonOverIntron;
+
+            struct {
+                bool yes=false;
+            } gene;
           
         } quant;
 
         //variation parameters
         struct {
-            bool yes;
+            bool yes=false;
             string vcfFile;
         } var;
 
         struct {
-            bool yes;
+            bool yes=false;
             bool SAMtag;
             string outputMode;
         } wasp;
