@@ -42,17 +42,7 @@ void ParametersSolo::initialize(Parameters *pPin)
     if (typeStr=="None") {
         type = SoloTypes::None;
         yes = false;
-        samAttrYes = false;
-        //solo SAM attributes not allowed
-        if ( pP->outSAMattrPresent.CR || pP->outSAMattrPresent.CY || pP->outSAMattrPresent.UR 
-          || pP->outSAMattrPresent.UY  || pP->outSAMattrPresent.CB  || pP->outSAMattrPresent.UB
-          || pP->outSAMattrPresent.sS  || pP->outSAMattrPresent.sQ  || pP->outSAMattrPresent.sM
-           ) {
-            ostringstream errOut;
-            errOut <<"EXITING because of FATAL INPUT ERROR: --outSAMattributes contains CR/CY/UR/UY/CB/UB tags which are not allowed for --soloType " << typeStr <<'\n';
-            errOut <<"SOLUTION: re-run STAR without these attribures\n";
-            exitWithError(errOut.str(), std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
-        };        
+        samAttrYes = false;     
         return;        
     } else if (typeStr=="CB_UMI_Simple" || typeStr=="Droplet") {
         type=SoloTypes::CB_UMI_Simple;        
@@ -195,17 +185,22 @@ void ParametersSolo::initialize(Parameters *pPin)
     //////////////////////////////////// features
     featureInd.fill(-1);
     featureYes.fill(false);
-    for (uint32 ii=0; ii<SoloFeatureTypes::Names.size(); ii++) {
-        for (auto &fin : featureIn) {
+    featureFirst = -1;
+    for (auto &fin : featureIn) {
+        for (uint32 ii=0; ii<SoloFeatureTypes::Names.size(); ii++) {
             if (fin==SoloFeatureTypes::Names[ii]) {
                 featureYes[ii]=true;
                 features.push_back(ii);
-                featureInd[ii]=features.size()-1;
+                if (featureFirst == -1)
+                    featureFirst = ii; //recored the first feature on the list
                 break;
             };
         };
     };
     nFeatures=features.size();
+    std::sort(features.begin(), features.end());
+    for (uint32 ii=0; ii<nFeatures; ii++)
+        featureInd[features[ii]]=ii;
 
     if (nFeatures != featureIn.size()) {
         ostringstream errOut;
@@ -225,20 +220,31 @@ void ParametersSolo::initialize(Parameters *pPin)
         exitWithError(errOut, std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
     };
     
-    if (type != SoloTypes::CB_samTagOut) {//gene quantification is needed
-        if (featureYes[SoloFeatureTypes::Gene]) {
-            pP->quant.gene.yes=true;
-            pP->quant.yes = true;
-        };
-        if (featureYes[SoloFeatureTypes::GeneFull]) {
-            pP->quant.geneFull.yes=true;
-            pP->quant.yes = true;
-        } else if (pP->outSAMattrPresent.GX || pP->outSAMattrPresent.GN) {//turn on quantification if no GeneFull or Gene, but GX/GN requested        
-            pP->quant.gene.yes=true;
-            pP->quant.yes = true;        
-        };
+    if (featureYes[SoloFeatureTypes::Gene]) {
+        pP->quant.gene.yes = true;
+        pP->quant.yes = true;
     };
+    if (featureYes[SoloFeatureTypes::GeneFull]) {
+        pP->quant.geneFull.yes = true;
+        pP->quant.yes = true;
+        if (!featureYes[SoloFeatureTypes::Gene]) 
+            pP->quant.gene.yes=false; //if GeneFull is requested, but Gene is not, turn it off - it could have been turned on because of GX/GN attributes
+    };
+    if (featureYes[SoloFeatureTypes::GeneFull_Ex50pAS]) {
+        pP->quant.geneFull_Ex50pAS.yes = true;
+        pP->quant.yes = true;
+        if (!featureYes[SoloFeatureTypes::Gene]) 
+            pP->quant.gene.yes=false; //if GeneFull is requested, but Gene is not, turn it off - it could have been turned on because of GX/GN attributes
+    };
+    if (featureYes[SoloFeatureTypes::GeneFull_ExonOverIntron]) {
+        pP->quant.geneFull_ExonOverIntron.yes = true;
+        pP->quant.gene.yes = true; //needed to prioritize exons over introns
+        pP->quant.yes = true;
+    };          
     
+    //initialize CB match to WL types
+    init_CBmatchWL();
+
     ////////////////////////////////////////////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////umiDedup
     umiDedup.initialize(this);
@@ -297,9 +303,9 @@ void ParametersSolo::initialize(Parameters *pPin)
                 };
             };
             if (cbWL.size()==0) {//empty whitelist
-            	exitWithError("EXITING because of FATAL ERROR: CB whitelist file " + soloCBwhitelist[0] + \
-            			       " is empty. \nSOLUTION: provide non-empty whitelist.\n" , \
-							   std::cerr, pP->inOut->logMain, EXIT_CODE_INPUT_FILES, *pP);
+                exitWithError("EXITING because of FATAL ERROR: CB whitelist file " + soloCBwhitelist[0] + \
+                               " is empty. \nSOLUTION: provide non-empty whitelist.\n" , \
+                               std::cerr, pP->inOut->logMain, EXIT_CODE_INPUT_FILES, *pP);
             };
         };
 
@@ -338,29 +344,7 @@ void ParametersSolo::initialize(Parameters *pPin)
         };
         
         umiV.extractPositionsFromString(umiPositionStr);
-        
-        /* debug
-        cbV[0].length=16;
-        cbV[0].anchorType=0;
-        cbV[0].anchorDist=0;
-        cbV[0].pos=0;
-        
-        umiV.length=10;
-        umiV.anchorType=0;
-        umiV.anchorDist=0;
-        umiV.pos=16;
-
-        //hard-coded inDrop
-        cbV.resize(2);
-        cbV[0].anchorType={0,2};
-        cbV[0].anchorDist={0,-1};//CB ends one base before the start of the anchor
-        cbV[1].anchorType={3,3};
-        cbV[1].anchorDist={1,8};//CB starts 1 base after the end of the anchor  
-        umiV.anchorType={3,3};
-        umiV.anchorDist={9,14};
-        ///////////////////
-        */
-        
+              
         umiV.adapterLength=adapterSeq.size();//one adapter for all
         cbWLsize=1;
         for (uint32 icb=0; icb<cbV.size(); icb++) {//cycle over WL files
@@ -382,7 +366,7 @@ void ParametersSolo::initialize(Parameters *pPin)
                 cbV[icb].wl.at(len1).push_back(cb1);
             };
             
-            cbV[icb].sortWhiteList();
+            cbV[icb].sortWhiteList(this);
             cbV[icb].wlFactor=cbWLsize;
             cbWLsize *= cbV[icb].totalSize;
         };
@@ -394,7 +378,7 @@ void ParametersSolo::initialize(Parameters *pPin)
     time(&rawTime);
     pP->inOut->logMain << timeMonthDayTime(rawTime) << " ... Finished reading, sorting and deduplicating CB whitelist sequences." <<endl;
 
-    //SAM attributes
+    //////////////////////////////////////////////////////////////SAM attributes
     samAttrYes=false;
     if ( (pP->outSAMattrPresent.CB || pP->outSAMattrPresent.UB) && type!=SoloTypes::CB_samTagOut) {
         samAttrYes=true;
@@ -405,39 +389,61 @@ void ParametersSolo::initialize(Parameters *pPin)
             exitWithError(errOut.str(),std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
         };
     } else if ( pP->outSAMattrPresent.UB && type==SoloTypes::CB_samTagOut) {
-    	exitWithError("EXITING because of fatal PARAMETERS error: UB attribute (corrected UMI) in --outSAMattributes cannot be used with --soloType CB_samTagOut \n" \
+        exitWithError("EXITING because of fatal PARAMETERS error: UB attribute (corrected UMI) in --outSAMattributes cannot be used with --soloType CB_samTagOut \n" \
                       "SOLUTION: instead, use UR (uncorrected UMI) in --outSAMattributes\n",   std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
     };
     
+    ////////////////////////////////////////////////////////////////readInfoYes: which feature is used to fill readInfo. Only one feature is allowed
     readInfoYes.fill(false);
-    if (samAttrYes) {//pSolo.samAttrFeature=0 by default, so need to check samAttrYes
-    	if (featureYes[SoloFeatureTypes::Gene]) {
-    		samAttrFeature=SoloFeatureTypes::Gene;
-    	} else if (featureYes[SoloFeatureTypes::GeneFull]) {//if Gene is not defined
-    		samAttrFeature=SoloFeatureTypes::GeneFull;
-    	} else {
+    if (featureYes[SoloFeatureTypes::VelocytoSimple] || featureYes[SoloFeatureTypes::Velocyto]) {//turn readInfo on for Gene needed by VelocytoSimple
+        readInfoYes[SoloFeatureTypes::Gene]=true;
+    };
+    samAttrFeature = featureFirst;
+    if (samAttrYes){//pSolo.samAttrFeature=0 by default, so need to check samAttrYes
+        if (   featureFirst == SoloFeatureTypes::Gene || featureFirst == SoloFeatureTypes::GeneFull ||
+               featureFirst == SoloFeatureTypes::GeneFull_Ex50pAS || featureFirst == SoloFeatureTypes::GeneFull_ExonOverIntron ) {
+            //all good
+        } else {
             ostringstream errOut;
-            errOut << "EXITING because of fatal PARAMETERS error: CB and/or UB attributes in --outSAMattributes require --soloFeatures Gene OR/AND GeneFull.\n";
-            errOut << "SOLUTION: re-run STAR adding Gene AND/OR GeneFull to --soloFeatures\n";
+            errOut << "EXITING because of fatal PARAMETERS error: CB and/or UB attributes in --outSAMattributes require --soloFeatures Gene OR/AND GeneFull OR/AND GeneFull_Ex50pAS.\n";
+            errOut << "SOLUTION: re-run STAR adding Gene AND/OR GeneFull OR/AND GeneFull_Ex50pAS OR/AND GeneFull_ExonOverIntron to --soloFeatures\n";
             exitWithError(errOut.str(),std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
-    	};
-
+        };
         readInfoYes[samAttrFeature]=true;
     };
-    if (featureYes[SoloFeatureTypes::VelocytoSimple] || featureYes[SoloFeatureTypes::Velocyto]) //turn readInfo on for Gene needed by VelocytoSimple
-        readInfoYes[SoloFeatureTypes::Gene]=true;    
+    readIndexYes = readInfoYes;
+
+    /////////////////////////////////////////////////////////////////// readFlag output
+    readStats.yes = false;
+    readStatsYes.fill(false);
+    if (readStats.type == "Standard") {
+        readStats.yes = true;
+        readStatsYes.fill(true);
+        readStatsYes[SoloFeatureTypes::VelocytoSimple] = false; //this could be allowed, but it will have the same info as Gene
+        readStatsYes[SoloFeatureTypes::Velocyto] = false;
+        readStatsYes[SoloFeatureTypes::SJ] = false; //output for SJ requires careful consideration for SoloReadFeature_record.cpp, for output of reads that do not have splice junction
+        for (uint32 ff=0; ff<readIndexYes.size(); ff++) {//merge with previous values
+            readIndexYes[ff] |= readStatsYes[ff];
+        };
+    } else if (readStats.type != "None" ) {
+        exitWithError("EXITING because of fatal PARAMETERS error: unrecognized option in --soloCellReadStats" + readStats.type
+                      + "\nSOLUTION: use allowed options: None OR Standard \n",
+                      std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
+    };
        
-    //umi filtering
-    umiFiltering.MultiGeneUMI=false;
-    umiFiltering.MultiGeneUMI_CR=false;
+    ///////////////////////////////////////////////////////////////////umi filtering
     if (umiFiltering.type[0]=="MultiGeneUMI") {
-        umiFiltering.MultiGeneUMI=true;
+        umiFiltering.MultiGeneUMI = true;
+        umiFiltering.yes = true;
+    } else if (umiFiltering.type[0]=="MultiGeneUMI_All") {
+        umiFiltering.MultiGeneUMI_All = true;
+        umiFiltering.yes = true;
     } else if (umiFiltering.type[0]=="MultiGeneUMI_CR") {
-        umiFiltering.MultiGeneUMI_CR=true;
+        umiFiltering.MultiGeneUMI_CR = true;
         if (umiDedup.typesIn.size()>1 || umiDedup.typesIn.at(0) != "1MM_CR")
             exitWithError("EXITING because of fatal PARAMETERS error: --soloUMIfiltering MultiGeneUMI_CR only works with --soloUMIdedup 1MM_CR"
-                          "\nSOLUTION: rerun with --soloUMIfiltering MultiGeneUMI_CR --soloUMIdedup 1MM_CR \n",
-                      std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
+                            "\nSOLUTION: rerun with --soloUMIfiltering MultiGeneUMI_CR --soloUMIdedup 1MM_CR \n",
+                            std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
     } else if (umiFiltering.type[0]=="-") {
         //nothing to do
     } else {
@@ -446,46 +452,22 @@ void ParametersSolo::initialize(Parameters *pPin)
                       std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
     };
     
-    {//CBmatchWL
-        if ( (typeStr=="CB_UMI_Complex" || typeStr=="CB_samTagOut") && (CBmatchWL.type!="Exact" && CBmatchWL.type!="1MM") ) {
-            ostringstream errOut;
-            errOut << "EXITING because of fatal PARAMETERS error: --soloCBmatchWLtype "<< CBmatchWL.type << " does not work with --soloType " << typeStr << "\n";
-            errOut << "SOLUTION: use allowed option: use --soloCBmatchWLtype Exact (exact matches only) OR 1MM (one match with 1 mismatched base)\n";
-            exitWithError(errOut.str(),std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
-        };
-        
-        
-        CBmatchWL.mm1 = false;
-        CBmatchWL.mm1_multi = false;
-        CBmatchWL.mm1_multi_pc = false;        
-        CBmatchWL.mm1_multi_Nbase = false;
-        CBmatchWL.oneExact = false; //if true, for a CB matching with 1 mismatch to a WL-CB, requires at least one other read to match this WL-CB. 
-                                  //this is true for all options except pseudocount
-            
-        if (CBmatchWL.type=="Exact") {    
-            CBmatchWL.oneExact=true;
-        } else if (CBmatchWL.type=="1MM") {
-            CBmatchWL.mm1=true;
-            CBmatchWL.oneExact=true;        
-        } else if (CBmatchWL.type=="1MM_multi") {
-            CBmatchWL.mm1=true;
-            CBmatchWL.mm1_multi=true;
-            CBmatchWL.oneExact=true;    
-        } else if (CBmatchWL.type=="1MM_multi_pseudocounts") {
-            CBmatchWL.mm1=true;
-            CBmatchWL.mm1_multi=true;
-            CBmatchWL.mm1_multi_pc=true;
-        } else if (CBmatchWL.type=="1MM_multi_Nbase_pseudocounts") {
-            CBmatchWL.mm1=true;
-            CBmatchWL.mm1_multi=true;
-            CBmatchWL.mm1_multi_pc=true;
-            CBmatchWL.mm1_multi_Nbase = true;
-        } else {
-            exitWithError("EXITING because of fatal PARAMETERS error: unrecognized option in --soloCBmatchWLtype " +CBmatchWL.type + "\nSOLUTION: use allowed options: Exact or 1MM or 1MM_multi or 1MM_multi_pseudocounts 1MM_multi_Nbase_pseudocounts\n",
-                        std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
-        };
-    };
+
     
+    //////////////////////////////////////////////////////////////////////////////////////////////
+    /////////////////////////////////////////////// MultiMappers
+    multiMap.initialize(this);
+    if (multiMap.yes.multi) {
+        if (type==SoloTypes::CB_samTagOut || type==SoloTypes::SmartSeq) {
+            exitWithError("EXITING because of fatal PARAMETERS error: multimapping options do not work for --soloType " +typeStr+ 
+                          "\nSOLUTION: use default option --soloMultiMappers Unique\n",
+                            std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
+        };
+        readIndexYes[SoloFeatureTypes::Gene]=true;
+        readIndexYes[SoloFeatureTypes::GeneFull]=true;
+        readIndexYes[SoloFeatureTypes::GeneFull_Ex50pAS]=true;
+        readIndexYes[SoloFeatureTypes::GeneFull_ExonOverIntron]=true;
+    };
 };
 
 /////////////////////////////////
@@ -501,28 +483,28 @@ void ParametersSolo::complexWLstrings() {
     cbWLstr.resize(cbWLsize);
     
     for (auto &cb : cbV) {//initialize
-        cb.icb=0;
-        cb.ilen=cb.minLen;
+        cb.iCB=0;
+        cb.iLen=cb.minLen;
     };
 
     for (uint32 ii=0; ii<cbWLsize; ii++) {//cycle over full WL
         for (uint64 ii=0; ii<cbV.size(); ii++) {//check for overflow and re-calculate all indexes
             SoloBarcode &cb=cbV[ii];
-            if (cb.icb == cb.wl[cb.ilen].size()) {//advance length
-                cb.ilen++;
-                cb.icb=0;//reset icb
+            if (cb.iCB == cb.wl[cb.iLen].size()) {//advance length
+                cb.iLen++;
+                cb.iCB=0;//reset iCB
             };
-            if (cb.ilen == cb.wl.size()){//advance barcode
-                cbV[ii+1].icb++;
-                cb.ilen=cb.minLen;//reset length
+            if (cb.iLen == cb.wl.size()){//advance barcode
+                cbV[ii+1].iCB++;
+                cb.iLen=cb.minLen;//reset length
             };
         };
         
         for (auto &cb : cbV) 
-            cbWLstr[ii] += convertNuclInt64toString(cb.wl[cb.ilen][cb.icb], cb.ilen) + "_";
+            cbWLstr[ii] += convertNuclInt64toString(cb.wl[cb.iLen][cb.iCB], cb.iLen) + "_";
         cbWLstr[ii].pop_back();
         
-        cbV[0].icb++;//shift by one for the next CB
+        cbV[0].iCB++;//shift by one for the next CB
     };
 };
 
@@ -594,7 +576,7 @@ void UMIdedup::initialize(ParametersSolo *pS)
         if (itype==tN) {//no match
             std::string tall;
             for (auto &t: typeNames)
-                tall +=" " + t; // concatenate allowd values
+                tall +=" " + t; // concatenate allowed values
             
             exitWithError("EXITING because of fatal PARAMETERS error: unrecognzied option --soloUMIdedup = " + typesIn[iin] + '\n'
                           + "SOLUTION: use allowed values: " + tall + '\n'
@@ -611,31 +593,107 @@ void UMIdedup::initialize(ParametersSolo *pS)
                     + "SOLUTION: use allowed options: Exact and/or NoDedup\n"
                     ,std::cerr, pS->pP->inOut->logMain, EXIT_CODE_PARAMETER, *pS->pP);            
     };
+       
+    //hard-coded for now
+    typeMain = types[0]; //main is the 0th entry in typesIn
+    countInd.main = 1;   //hard-coded - 1 column is always main
+};
+
+void MultiMappers::initialize(ParametersSolo* pS)
+{
+    yes.N = 0;
+    countInd.I.fill((uint32) -1); //marks types not used
+    yes.B.fill(false);
+
+    for (uint32 iin=0; iin<typesIn.size(); iin++) {
+        uint32 itype;
+        for (itype=0; itype<tN; itype++) {
+            if (typesIn[iin] == typeNames[itype])
+                break; //found match
+        };
+        
+        if (itype==tN) {//no match
+            std::string tall;
+            for (auto &t: typeNames)
+                tall +=" " + t; // concatenate allowed values
+            
+            exitWithError("EXITING because of fatal PARAMETERS error: unrecognzied option --soloMultiMappers = " + typesIn[iin] + '\n'
+                          + "SOLUTION: use allowed values: " + tall + '\n'
+                          ,std::cerr, pS->pP->inOut->logMain, EXIT_CODE_PARAMETER, *pS->pP);
+        };
+        
+        if (itype == typeI::Unique)
+            continue; //Unique type does not have to be recorded TODO use it to perform filtering and stats
+
+        
+        types.push_back(itype);
+        yes.B[itype] = true;
+        yes.N++;
+    };
     
-//     if (yes.CR && (yes.N>1)) {//TODO remove this limitation
-//             exitWithError("EXITING because of fatal PARAMETERS error: --soloUMIdedup 1MM_CR is not allowed in combination with other UMI deduplication options\n"
-//                       "SOLUTION: specify --soloUMIdedup 1MM_CR only, or a combination of Exact and/or 1MM_All and/or 1MM_Directional"
-//                       ,std::cerr, pS->pP->inOut->logMain, EXIT_CODE_PARAMETER, *pS->pP); 
-//     };
+    if (yes.N==0) {//only Unique, no multimappers
+        yes.multi=false;
+        return;
+    };
+    
+    uint32 ind1=1; //start
+    for (const auto &itype : types) {
+        countInd.I[itype] = ind1;
+        ind1 += pS->umiDedup.yes.N;
+    };
     
     //hard-coded for now
     typeMain = types[0]; //main is the 0th entry in typesIn
     countInd.main = 1;   //hard-coded - 1 column is always main
     
-    /*        
-        countInd.N = 1;
-        countInd.CR = 1; //starts from 1, since 0 is the gene
-        
-    } else if (pS->type == pS->SoloTypes::SmartSeq) {//this is fixed for now
-        countInd.N = 2;
-        countInd.Exact = 1; //starts from 1, since 0 is the gene
-        countInd.NoDedup = 2;
-    } else {
-        countInd.N = 3;
-        countInd.Exact = 1; //starts from 1, since 0 is the gene
-        countInd.All = 2;
-        countInd.Directional = 3;
+    yes.multi = yes.Uniform | yes.Rescue | yes.PropUnique | yes.EM;
+};
+
+////////////////////////////////////////////////////
+void ParametersSolo::init_CBmatchWL()
+{//CBmatchWL
+    bool incomp1 =        typeStr=="CB_UMI_Complex" && (CBmatchWL.type!="Exact" && CBmatchWL.type!="1MM" && CBmatchWL.type!="EditDist_2");
+    incomp1 = incomp1 || (typeStr=="CB_samTagOut"   && (CBmatchWL.type!="Exact" && CBmatchWL.type!="1MM"));
+    incomp1 = incomp1 || (typeStr!="CB_UMI_Complex" &&  CBmatchWL.type=="EditDist_2");
+    
+    if ( incomp1 ) {
+        ostringstream errOut;
+        errOut << "EXITING because of fatal PARAMETERS error: --soloCBmatchWLtype "<< CBmatchWL.type << " does not work with --soloType " << typeStr << "\n";
+        errOut << "SOLUTION: use allowed option: use --soloCBmatchWLtype Exact (exact matches only) OR 1MM (one match with 1 mismatched base)\n";
+        exitWithError(errOut.str(),std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
     };
-    countInd.main = countInd.I[typeMain];
-    */
-};
\ No newline at end of file
+    
+    
+    CBmatchWL.mm1 = false;
+    CBmatchWL.mm1_multi = false;
+    CBmatchWL.mm1_multi_pc = false;        
+    CBmatchWL.mm1_multi_Nbase = false;
+    CBmatchWL.oneExact = false; //if true, for a CB matching with 1 mismatch to a WL-CB, requires at least one other read to match this WL-CB. 
+                                //this is true for all options except pseudocount
+    CBmatchWL.EditDist_2 = false;
+        
+    if (CBmatchWL.type=="Exact") {    
+        CBmatchWL.oneExact=true;
+    } else if (CBmatchWL.type=="1MM") {
+        CBmatchWL.mm1=true;
+        CBmatchWL.oneExact=true;        
+    } else if (CBmatchWL.type=="1MM_multi") {
+        CBmatchWL.mm1=true;
+        CBmatchWL.mm1_multi=true;
+        CBmatchWL.oneExact=true;    
+    } else if (CBmatchWL.type=="1MM_multi_pseudocounts") {
+        CBmatchWL.mm1=true;
+        CBmatchWL.mm1_multi=true;
+        CBmatchWL.mm1_multi_pc=true;
+    } else if (CBmatchWL.type=="1MM_multi_Nbase_pseudocounts") {
+        CBmatchWL.mm1=true;
+        CBmatchWL.mm1_multi=true;
+        CBmatchWL.mm1_multi_pc=true;
+        CBmatchWL.mm1_multi_Nbase = true;
+    } else if (CBmatchWL.type=="EditDist_2") {
+        CBmatchWL.EditDist_2=true;
+    } else {
+        exitWithError("EXITING because of fatal PARAMETERS error: unrecognized option in --soloCBmatchWLtype " +CBmatchWL.type + "\nSOLUTION: use allowed options: Exact or 1MM or 1MM_multi or 1MM_multi_pseudocounts 1MM_multi_Nbase_pseudocounts\n",
+                        std::cerr, pP->inOut->logMain, EXIT_CODE_PARAMETER, *pP);
+    };
+};
