@@ -49,16 +49,17 @@ void Parameters::openReadsFiles()
 
             for (uint32 ifile=0; ifile<readFilesN; ifile++) {
                 
-                system(("ls -lL " + readFilesNames[imate][ifile] + " > "+ outFileTmp+"/readFilesIn.info 2>&1").c_str());
+                if ( system(("ls -lL " + readFilesNames[imate][ifile] + " > "+ outFileTmp+"/readFilesIn.info 2>&1").c_str()) !=0 )
+                    warningMessage(" Could not ls " + readFilesNames[imate][ifile], std::cerr, inOut->logMain, *this);
 
                 ifstream readFilesIn_info((outFileTmp+"/readFilesIn.info").c_str());
                 inOut->logMain <<readFilesIn_info.rdbuf();
 
                 {//try to open the files - throw an error if a file cannot be opened
-					ifstream rftry(readFilesNames.at(imate).back().c_str());
+					ifstream rftry(readFilesNames[imate][ifile].c_str());
 					if (!rftry.good()){
 						exitWithError("EXITING: because of fatal INPUT file error: could not open read file: " + \
-									   readFilesNames.at(imate).back() + \
+									   readFilesNames[imate][ifile] + \
 									   "\nSOLUTION: check that this file exists and has read permision.\n", \
 									   std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
 					};
