@@ -13,18 +13,18 @@
 #include "ChimericDetection.h"
 #include "SoloRead.h"
 #include "ReadAnnotations.h"
+#include "SpliceGraph.h"
+#include "ClipMate.h"
 
 #include <time.h>
 #include <random>
 
 class ReadAlign {
     public:
-         //methods
         ReadAlign (Parameters& Pin, Genome &genomeIn, Transcriptome *TrIn, int iChunk);//allocate arrays
         int oneRead();
 
-        //vars
-        Genome &mapGen; //mapped-to-genome structure
+        Genome &mapGen, &genOut; //mapped-to-genome structure
 
         uint64 iRead, iReadAll;
         char **Read1;
@@ -42,9 +42,36 @@ class ReadAlign {
 
         uint outFilterMismatchNmaxTotal;
         uint Lread, readLength[MAX_N_MATES], readLengthOriginal[MAX_N_MATES], readLengthPair, readLengthPairOriginal;
+        string readBarcodeSeq, readBarcodeQual; //Solo barcode sequence
+        
         intScore maxScoreMate[MAX_N_MATES];
 
-        uint readFilesIndex;
+        uint32 readFilesIndex;
+        
+        //quality histogram
+        array<array<uint64,256>,MAX_N_MATES> qualHist;
+        
+        //transcripts (aligns)
+        uint nW;
+        uint *nWinTr; //number of recorded transcripts per window
+        Transcript trA, trA1, *trInit; //transcript, best tr, next best tr, initialized tr
+        Transcript ***trAll; //all transcripts for all windows
+        Transcript *trArray; //linear array of transcripts to store all of them from all windows
+        Transcript **trArrayPointer; //linear array of transcripts to store all of them from all windows
+
+        uint64 nTr; // number of transcripts called
+        Transcript *trMult[MAX_N_MULTMAP];//selected alignments - to the mapGen        
+        Transcript *trBest;//bset transcripts
+        
+        struct {
+            bool yes;
+            uint64 alN;
+            Transcript **alMult;//multimapping transcripts - transformed to output genome
+            Transcript *alBest;//best align
+        } alignsGenOut;
+        
+        
+        Transcript *alignTrAll;//alignments to transcriptome        
 
         ReadAlign *waspRA; //ReadAlign for alternative WASP alignment
         int waspType, waspType1; //alignment ASE-WASP type and
@@ -52,8 +79,18 @@ class ReadAlign {
         ReadAlign *peMergeRA; //ReadAlign for merged PE mates
 
         ChimericDetection *chimDet;
+        void peOverlapChimericSEtoPE(const Transcript *seTrIn1, const Transcript *seTrIn2, Transcript *peTrOut1, Transcript *peTrOut2);
 
         SoloRead *soloRead; //counts reads per CB per and outputs CB/UMI/gene into file, per thread
+        
+        SpliceGraph *splGraph;
+        
+        vector<vector<ClipMate>> clipMates;
+
+	//input,output
+        char** outBAMoneAlign;
+        uint* outBAMoneAlignNbytes;
+        int alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint trChrStart, uint mateChr, uint mateStart, char mateStrand, int unmapType, bool *mateMap, vector<int> outSAMattrOrder, char** outBAMarray, uint* outBAMarrayN);
 
     private:
         Parameters& P; //pointer to the parameters, will be initialized on construction
@@ -70,28 +107,19 @@ class ReadAlign {
 
         //input,output
 
-        char** outBAMoneAlign;
-        uint* outBAMoneAlignNbytes;
-
         ostringstream samStreamCIGAR, samStreamSJmotif, samStreamSJintron;
         vector <string> matesCIGAR;
 
         intScore *scoreSeedToSeed, *scoreSeedBest;
         uint *scoreSeedBestInd, *seedChain, *scoreSeedBestMM;
 
-        bool outFilterPassed; //true if alignment passed all filter and is output to SAM/BAM
-
-//         StatsAll *statsRA;
-
-        //transcript
-        Transcript* trArray; //linear array of transcripts to store all of them from all windows
-        Transcript** trArrayPointer; //linear array of transcripts to store all of them from all windows
+        bool outFilterBySJoutPass; //true if alignment passed all filters and is output
+        bool outSAMfilterPass; //true if alignment passed SAM filter
 
         //read
         uint64 iMate;
         char readFilter; //Illumina not passed Y/N
         bool revertStrand; //what to do with the strand, according to strandType and iMate
-        uint clip3pNtotal[MAX_N_MATES], clip5pNtotal[MAX_N_MATES], clip3pAdapterN[MAX_N_MATES]; //total number of trimmed bases from 5p,3p
         int readFileType; //file type: 1=fasta; 2=fastq
 
         vector<string>readNameExtra;
@@ -101,7 +129,6 @@ class ReadAlign {
         char** Qual0;
         char** readNameMates;
         char* readName;
-        char** Qual1; //modified QSs for scoring
 
         uint readNmates;
         //split
@@ -121,7 +148,7 @@ class ReadAlign {
         int unmapType; //marker for why a read is unmapped
 
         uint mapMarker; //alignment marker (typically, if there is something wrong)
-        uint nA, nP, nW, nWall, nUM[2]; //number of all alignments,  pieces, windows, U/M,
+        uint nA, nP, nWall, nUM[2]; //number of all alignments,  pieces, windows, U/M,
         uint *nWA, *nWAP, *WALrec, *WlastAnchor; //number of alignments per window, per window per piece, min recordable length per window
         bool *WAincl; //alginment inclusion mask
 
@@ -129,13 +156,9 @@ class ReadAlign {
         char *swT;
 
         uint storedLmin, uniqLmax, uniqLmaxInd, multLmax, multLmaxN, multNmin, multNminL, multNmax, multNmaxL;
-        uint nTr, nTrMate; // number of transcripts called
         intScore maxScore;//maximum alignment score
-
-        Transcript trA, trA1, *trBest, *trInit; //transcript, best tr, next best tr, initialized tr
-        Transcript ***trAll; //all transcripts for all windows
-        uint *nWinTr; //number of recorded transcripts per window
-
+        bool mateMapped[2];
+        
         //old chimeric detection
         uint chimN, chimRepeat, chimStr;
         int chimMotif;
@@ -144,11 +167,6 @@ class ReadAlign {
         //new chimeric detection
         bool chimRecord; //true if chimeric aligment was detected
 
-        Transcript *alignC, *extendC, *polyAtailC; //alignment rules/conditions
-
-        Transcript* trMult[MAX_N_MULTMAP];//multimapping transcripts
-        Transcript *alignTrAll;//alignments to transcriptome
-
         struct {
             bool yes;
             uint nOv;//number of overlapping bases
@@ -163,12 +181,16 @@ class ReadAlign {
         void resetN();//resets the counters to 0
         void multMapSelect();
         int mapOneRead();
+        void mapOneReadSpliceGraph();
         uint maxMappableLength2strands(uint pieceStart, uint pieceLength, uint iDir, uint iSA1, uint iSA2, uint& maxL, uint iFrag);
         void storeAligns (uint iDir, uint Shift, uint Nrep, uint L, uint indStartEnd[2], uint iFrag);
 
         bool outputTranscript(Transcript *trOut, uint nTrOut, ofstream *outBED);
-        uint outputTranscriptSAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint mateChr, uint mateStart, char mateStrand, int unmapType, bool *mateMapped, ostream *outStream);
-        int alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint trChrStart, uint mateChr, uint mateStart, char mateStrand, int unmapType, bool *mateMapped, vector<int> outSAMattrOrder, char** outBAMarray, uint* outBAMarrayN);
+
+        uint64 outputTranscriptSAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint mateChr, uint mateStart, char mateStrand, int unmapType, bool *mateMap, ostream *outStream);
+
+        uint64 outputSpliceGraphSAM(Transcript const &trOut, uint nTrOut, uint iTrOut, ostream *outStream);
+
         void samAttrNM_MD (Transcript const &trOut, uint iEx1, uint iEx2, uint &tagNM, string &tagMD);
 
         void outputTranscriptSJ(Transcript const &trOut, uint nTrOut, OutSJ *outStream, uint sjReadStartN );
@@ -182,8 +204,8 @@ class ReadAlign {
         void chimericDetectionOldOutput();
         bool chimericDetectionMult();
         void chimericDetectionPEmerged(ReadAlign &seRa);
-//         void chimericDetectionPEmergedTrim();
 
+        void transformGenome();
         void outputAlignments();
         void calcCIGAR(Transcript const &trOut, uint nMates, uint iExMate, uint leftMate);
 
@@ -197,7 +219,14 @@ class ReadAlign {
         void peOverlapMergeMap();
         void peMergeMates();
         void peOverlapSEtoPE(ReadAlign &seRA);
-
+        
+        //output alignments functions
+        void outFilterBySJout();
+        void outReadsUnmapped();
+        void spliceGraphWriteSAM();
+        void alignedAnnotation();
+        void writeSAM(uint64 nTrOutSAM, Transcript **trOutSAM, Transcript *trBestSAM);
+        void recordSJ(uint64 nTrO, Transcript **trO, OutSJ *cSJ);
 
 };
 
