@@ -11,12 +11,13 @@ void ReadAlign::samAttrNM_MD (Transcript const &trOut, uint iEx1, uint iEx2, uin
     tagMD="";
     char* R=Read1[trOut.roStr==0 ? 0:2];
     uint matchN=0;
+    uint32 nMM=0, nI=0, nD=0;
     for (uint iex=iEx1;iex<=iEx2;iex++) {
         for (uint ii=0;ii<trOut.exons[iex][EX_L];ii++) {
             char r1=R[ii+trOut.exons[iex][EX_R]];
-            char g1=mapGen.G[ii+trOut.exons[iex][EX_G]];
+            char g1=genOut.G[ii+trOut.exons[iex][EX_G]];
             if ( r1!=g1 || r1==4 || g1==4) {
-                ++tagNM;
+                ++nMM;
                 tagMD+=to_string(matchN);
                 tagMD+=P.genomeNumToNT[(uint8) g1];
                 matchN=0;
@@ -25,22 +26,25 @@ void ReadAlign::samAttrNM_MD (Transcript const &trOut, uint iEx1, uint iEx2, uin
             };
         };
         if (iex<iEx2) {
-            if (trOut.canonSJ[iex]==-1) {//deletion
-                tagNM+=trOut.exons[iex+1][EX_G]-(trOut.exons[iex][EX_G]+trOut.exons[iex][EX_L]);
+            if (trOut.canonSJ[iex] < 0) {//indels, junction are not included in edit distance
+                nD+=trOut.exons[iex+1][EX_G]-(trOut.exons[iex][EX_G]+trOut.exons[iex][EX_L]);//deletion
+            };
+            nI+=trOut.exons[iex+1][EX_R]-trOut.exons[iex][EX_R]-trOut.exons[iex][EX_L];//insertion
+            if (trOut.canonSJ[iex]==-1) {//deletion. TODO: This does not work if there is both deletion and insertion!!!
                 tagMD+=to_string(matchN) + "^";
                 for (uint ii=trOut.exons[iex][EX_G]+trOut.exons[iex][EX_L];ii<trOut.exons[iex+1][EX_G];ii++) {
-                    tagMD+=P.genomeNumToNT[(uint8) mapGen.G[ii]];
+                    tagMD+=P.genomeNumToNT[(uint8) genOut.G[ii]];
                 };
                 matchN=0;
-            } else if (trOut.canonSJ[iex]==-2) {//insertion
-                tagNM+=trOut.exons[iex+1][EX_R]-trOut.exons[iex][EX_R]-trOut.exons[iex][EX_L];
             };
         };
     };
     tagMD+=to_string(matchN);
+    //cout << nMM<<" "<<nD<<" "<<nI<<endl;
+    tagNM=nMM+nI+nD;
 };
 
-int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint trChrStart, uint mateChr, uint mateStart, char mateStrand, int alignType, bool *mateMapped, vector<int> outSAMattrOrder, char** outBAMarray, uint* outBAMarrayN) {
+int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint trChrStart, uint mateChr, uint mateStart, char mateStrand, int alignType, bool *mateMap, vector<int> outSAMattrOrder, char** outBAMarray, uint* outBAMarrayN) {
     //return: number of lines (mates)
 
     //alignType>=0: unmapped reads
@@ -63,7 +67,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
     uint16 samFLAG=0;
 
 
-    bool flagPaired = P.readNmates==2;
+    bool flagPaired = P.readNmates==2; //not readNends: this is about alignments
 
     uint nMates=1;
     if (alignType<0) {//mapped reads: SAM
@@ -92,7 +96,16 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         calcCIGAR(trOut, nMates, iExMate, leftMate);
     };
 
-    for (uint imate=0;imate < (alignType<0 ? nMates:P.readNmates);imate++) {
+    int32 gxgnGene = -1; //gene to output in GX/GN
+    if (P.outSAMattrPresent.GX || P.outSAMattrPresent.GN) {
+        auto annFeat = readAnnot.annotFeatures[P.pSolo.samAttrFeature];
+        if (annFeat.fSet.size()==1 && iTrOut < annFeat.fAlign.size() && annFeat.fAlign[iTrOut].size()==1) {
+            //                        2nd condition needed when this function is called to output transcriptomic alignments, where GX/GN are not allowed, and iTrOut can be large
+            gxgnGene = *annFeat.fAlign[iTrOut].begin();
+        };
+    };
+
+    for (uint imate=0;imate < (alignType<0 ? nMates:P.readNmates);imate++) { //not readNends: this is about alignments
 
         uint iEx1=0;
         uint iEx2=0;
@@ -106,17 +119,17 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         uint trimL1=0, trimR1=0;
 
         if (alignType>=0) {//this mate is unmapped
-            if (mateMapped!=NULL && mateMapped[imate]) continue; //this mate was mapped, do not record it as unmapped
+            if (mateMap!=NULL && mateMap[imate]) continue; //this mate was mapped, do not record it as unmapped
             samFLAG=0x4;
-            if (P.readNmates==2) {//paired read
+            if (P.readNmates==2) {//paired read, //not readNends: this is about alignments
                 samFLAG|=0x1 + (imate==0 ? 0x40 : 0x80);
-                if (mateMapped[1-imate]) {//mate mapped
+                if (mateMap[1-imate]) {//mate mapped
                     if (trOut.Str!=(1-imate))
                     {//mate strand reverted
                        samFLAG|=0x20;
                     };
                     mateChr=trOut.Chr;
-                    trChrStart=mapGen.chrStart[mateChr];
+                    trChrStart=genOut.chrStart[mateChr];
                     mateStart=trOut.exons[0][EX_G] - trChrStart;
                     mateStrand= trOut.Str == (1-imate) ? 0 : 1;
 
@@ -132,7 +145,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
 
             if (readFilter=='Y') samFLAG|=0x200; //not passing quality control
 
-            if (mateMapped[1-imate])
+            if (mateMap[1-imate])
             {//mate is mapped, fill the infromation from trOut
 
             };
@@ -165,12 +178,16 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                 attrN+=bamAttrArrayWrite(soloRead->readBar->bSeq,"sS",attrOutArray+attrN);
             if (P.outSAMattrPresent.sQ)
                 attrN+=bamAttrArrayWrite(soloRead->readBar->bQual,"sQ",attrOutArray+attrN);
+            if (P.outSAMattrPresent.cN) {
+                vector <int32> v1={(int32)clipMates[imate][0].clippedN, (int32)clipMates[imate][1].clippedN};
+                attrN+=bamAttrArrayWrite(v1,"cN",attrOutArray+attrN);
+            };
             
         } else {//this mate is mapped
             if (flagPaired) {//paired reads
                 samFLAG=0x0001;
                 if (iExMate==trOut.nExons-1) {//single mate
-                    if (mateChr>mapGen.nChrReal) samFLAG|=0x0008; //not mapped as pair
+                    if (mateChr>genOut.nChrReal) samFLAG|=0x0008; //not mapped as pair
                 } else {//properly paired
                     samFLAG|=0x0002; //mapped as pair
                 };
@@ -180,11 +197,8 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
 
             if (readFilter=='Y') samFLAG|=0x200; //not passing quality control
 
-            if (alignType==-11 || alignType==-12 || alignType==-13) {
-                samFLAG|=0x800; //mark chimeric alignments
-            } else {//only non-chimeric alignments will be marked as non-primary, since chimeric are already marked with 0x800
-                if (!trOut.primaryFlag) samFLAG|=0x100;//mark not primary align
-            };
+            if (alignType==-11 || alignType==-12 || alignType==-13) samFLAG|=0x800; //mark chimeric alignments
+            if (!trOut.primaryFlag) samFLAG|=0x100; //mark not primary align
 
             iEx1 = (imate==0 ? 0 : iExMate+1);
             iEx2 = (imate==0 ? iExMate : trOut.nExons-1);
@@ -207,13 +221,13 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
 
             uint trimL;
             if (Str==0 && Mate==0) {
-                trimL=clip5pNtotal[Mate];
+                trimL=clipMates[Mate][0].clippedN;
             } else if (Str==0 && Mate==1) {
-                trimL=clip3pNtotal[Mate];
+                trimL=clipMates[Mate][1].clippedN;
             } else if (Str==1 && Mate==0) {
-                trimL=clip3pNtotal[Mate];
+                trimL=clipMates[Mate][1].clippedN;
             } else {
-                trimL=clip5pNtotal[Mate];
+                trimL=clipMates[Mate][0].clippedN;
             };
 
             nCIGAR=0; //number of CIGAR operations
@@ -232,20 +246,19 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                     uint gapR=trOut.exons[ii][EX_R]-trOut.exons[ii-1][EX_R]-trOut.exons[ii-1][EX_L];
                     //it's possible to have a D or N and I at the same time
                     if (gapR>0){
-
                         packedCIGAR[nCIGAR++]=gapR<<BAM_CIGAR_OperationShift | BAM_CIGAR_I;
                     };
                     if (trOut.canonSJ[ii-1]>=0 || trOut.sjAnnot[ii-1]==1) {//junction: N
-
                         packedCIGAR[nCIGAR++]=gapG<<BAM_CIGAR_OperationShift | BAM_CIGAR_N;
                         SJmotif.push_back(trOut.canonSJ[ii-1] + (trOut.sjAnnot[ii-1]==0 ? 0 : SJ_SAM_AnnotatedMotifShift)); //record junction type
                         SJintron.push_back((int32) (trOut.exons[ii-1][EX_G] + trOut.exons[ii-1][EX_L] + 1 - trChrStart) );//record intron start
                         SJintron.push_back((int32) (trOut.exons[ii][EX_G] - trChrStart)); //record intron end
-                    } else if (gapG>0) {//deletion: N
+                    } else if (gapG>0) {//deletion: D
                         packedCIGAR[nCIGAR++]=gapG<<BAM_CIGAR_OperationShift | BAM_CIGAR_D;
                     };
                 };
-                packedCIGAR[nCIGAR++]=trOut.exons[ii][EX_L]<<BAM_CIGAR_OperationShift | BAM_CIGAR_M;
+                if (trOut.exons[ii][EX_L]>0) //do not record 0-length blocks (which could appear near junctions followed by deletions)
+                    packedCIGAR[nCIGAR++]=trOut.exons[ii][EX_L]<<BAM_CIGAR_OperationShift | BAM_CIGAR_M;
             };
 
             if (SJmotif.size()==0) {//no junctions recorded, mark with -1
@@ -301,6 +314,13 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                             attrN+=bamAttrArrayWrite('-',"XS",attrOutArray+attrN);
                         };
                         break;
+                        
+                    case ATTR_cN:
+                        {
+                            vector <int32> v1={(int32)clipMates[imate][0].clippedN, (int32)clipMates[imate][1].clippedN};
+                            attrN+=bamAttrArrayWrite(v1,"cN",attrOutArray+attrN);
+                            break;
+                        };
                     case ATTR_NM:
                         if ( tagNM == (uint) -1 ) samAttrNM_MD (trOut, iEx1, iEx2, tagNM, tagMD);
                         attrN+=bamAttrArrayWriteInt(tagNM,"NM",attrOutArray+attrN,P);
@@ -318,8 +338,8 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                             for (uint ii=iEx1;ii<=iEx2;ii++) {
                                 rb.push_back( (int32) trOut.exons[ii][EX_R]+1 );
                                 rb.push_back( (int32) trOut.exons[ii][EX_R]+trOut.exons[ii][EX_L]);
-                                rb.push_back( (int32) (trOut.exons[ii][EX_G]-mapGen.chrStart[trOut.Chr]+1) );
-                                rb.push_back( (int32) (trOut.exons[ii][EX_G]-mapGen.chrStart[trOut.Chr]+trOut.exons[ii][EX_L]) );
+                                rb.push_back( (int32) (trOut.exons[ii][EX_G]-genOut.chrStart[trOut.Chr]+1) );
+                                rb.push_back( (int32) (trOut.exons[ii][EX_G]-genOut.chrStart[trOut.Chr]+trOut.exons[ii][EX_L]) );
                             };
                             attrN+=bamAttrArrayWrite(rb,"rB",attrOutArray+attrN);
                         };
@@ -371,14 +391,48 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                     case ATTR_UY:
                         attrN+=bamAttrArrayWrite(soloRead->readBar->umiQual,"UY",attrOutArray+attrN);
                         break;
+
                     case ATTR_GX:
-                        if (readAnnot.geneConcordant.size()==1 && trOut.alignGenes.size()==1) //only output if read maps to a single gene
-                            attrN+=bamAttrArrayWrite(chunkTr->geID[*trOut.alignGenes.begin()],"GX",attrOutArray+attrN);
+                        {
+                            string algenes = gxgnGene<0 ? "-" : chunkTr->geID[gxgnGene];
+                            attrN+=bamAttrArrayWrite(algenes, "GX",attrOutArray+attrN);
+                        };
                         break;
+                            
                     case ATTR_GN:
-                        if (readAnnot.geneConcordant.size()==1 && trOut.alignGenes.size()==1) //only output if read maps to a single gene
-                            attrN+=bamAttrArrayWrite(chunkTr->geName[*trOut.alignGenes.begin()],"GN",attrOutArray+attrN);
-                        break;                        
+                        {
+                            string algenes = gxgnGene<0 ? "-" : chunkTr->geName[gxgnGene];
+                            attrN+=bamAttrArrayWrite(algenes, "GN",attrOutArray+attrN);
+                        };
+                        break;       
+
+                    case ATTR_gx:
+                        {
+                            string algenes;
+                            for (auto &gg: readAnnot.annotFeatures[P.pSolo.samAttrFeature].fAlign[iTrOut])
+                                algenes += chunkTr->geID[gg] + ";";
+                            if (algenes.size()>0) {
+                                algenes.pop_back();
+                            } else {
+                                algenes = "-";
+                            };
+                            attrN+=bamAttrArrayWrite(algenes,"gx",attrOutArray+attrN);                            
+                        };
+                        break;
+
+                    case ATTR_gn:
+                        {
+                            string algenes;
+                            for (auto &gg: readAnnot.annotFeatures[P.pSolo.samAttrFeature].fAlign[iTrOut])
+                                algenes += chunkTr->geName[gg] + ";";
+                            if (algenes.size()>0) {
+                                algenes.pop_back();
+                            } else {
+                                algenes = "-";
+                            };
+                            attrN+=bamAttrArrayWrite(algenes,"gn",attrOutArray+attrN);                            
+                        };
+                        break;                                                       
                         
                     case ATTR_sM:
                         attrN+=bamAttrArrayWrite(soloRead->readBar->cbMatch,"sM",attrOutArray+attrN);
@@ -390,12 +444,11 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
                         attrN+=bamAttrArrayWrite(soloRead->readBar->bQual,"sQ",attrOutArray+attrN);
                         break;
                         
-                    //following attributes are not processed here
                     case ATTR_CB:
-                        if (soloRead->readBar->cbSeqCorrected!="")
-                        attrN+=bamAttrArrayWrite(soloRead->readBar->cbSeqCorrected,"CB",attrOutArray+attrN);                        
+                        if (soloRead->readBar->cbSeqCorrected!="") //only if corrected CB is defined (e.g. CB_samTagOut), otherwise it will be added at the BAM sorting stage
+                        	attrN+=bamAttrArrayWrite(soloRead->readBar->cbSeqCorrected,"CB",attrOutArray+attrN);
                         break;
-                    case ATTR_UB:
+                    case ATTR_UB: //will be added at the BAM sorting stage
                         break;
                         
                     default:
@@ -407,10 +460,8 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
             };
         };
 
-        if (P.readFilesTypeN==10) {
-//             if (readNameExtra[Mate].size()<1)
-//                 cout << iReadAll <<" " <<readName <<endl;
-            attrN+=bamAttrArrayWriteSAMtags(readNameExtra[Mate],attrOutArray+attrN);
+        if (P.readFilesTypeN==10 && !P.readFiles.samAttrKeepNone) {
+            attrN+=bamAttrArrayWriteSAMtags(readNameExtra[Mate], attrOutArray+attrN, P);
         };
 ////////////////////////////// prepare sequence and qualities
         char seqMate[DEF_readSeqLengthMax+1], qualMate[DEF_readSeqLengthMax+1];
@@ -476,7 +527,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         //6: next refID Ref-ID of the next segment (􀀀1  mate refID < n ref)
         if (nMates>1) {
             pBAM[6]=trOut.Chr;
-        } else if (mateChr<mapGen.nChrReal){
+        } else if (mateChr<genOut.nChrReal){
             pBAM[6]=mateChr;
         } else {
             pBAM[6]=-1;
@@ -485,7 +536,7 @@ int ReadAlign::alignBAM(Transcript const &trOut, uint nTrOut, uint iTrOut, uint
         //7: next pos 0-based leftmost pos of the next segment (= PNEXT 􀀀 1)
         if (nMates>1) {
             pBAM[7]=trOut.exons[(imate==0 ? iExMate+1 : 0)][EX_G] - trChrStart;
-        } else if (mateChr<mapGen.nChrReal){
+        } else if (mateChr<genOut.nChrReal){
             pBAM[7]=mateStart;
         } else {
             pBAM[7]=-1;
