@@ -1,28 +1,38 @@
 #include "ReadAlign.h"
 #include "readLoad.h"
+#include "readBarcodeLoad.h"
 #include "SequenceFuns.h"
 #include "ErrorWarning.h"
+#include "GlobalVariables.h"
 
 int ReadAlign::oneRead() {//process one read: load, map, write
 
     //load read name, sequence, quality from the streams into internal arrays
-    int readStatus[2];
+    int readStatus[P.readNends];
 
-
-    readStatus[0]=readLoad(*(readInStream[0]), P, 0, readLength[0], readLengthOriginal[0], readNameMates[0], Read0[0], Read1[0], Qual0[0], Qual1[0], clip3pNtotal[0], clip5pNtotal[0], clip3pAdapterN[0], iReadAll, readFilesIndex, readFilter, readNameExtra[0]);
-    if (P.readNmates==2) {//read the 2nd mate
-        readStatus[1]=readLoad(*(readInStream[1]), P, 1, readLength[1], readLengthOriginal[1], readNameMates[1], Read0[1], Read1[0]+readLength[0]+1, Qual0[1], Qual1[0]+readLength[0]+1, clip3pNtotal[1], clip5pNtotal[1], clip3pAdapterN[1], iReadAll, readFilesIndex, readFilter, readNameExtra[1]);
-
-        if (readStatus[0]!=readStatus[1]) {
+    for (uint32 im=0; im<P.readNends; im++) {
+        readStatus[im] = readLoad(*(readInStream[im]), P, readLength[im], readLengthOriginal[im], readNameMates[im], Read0[im], Read1[im], Qual0[im], clipMates[im], iReadAll, readFilesIndex, readFilter, readNameExtra[im]);
+        if (readStatus[im] != readStatus[0]) {//check if the end of file was reached or not for all files
             ostringstream errOut;
-            errOut << "EXITING because of FATAL ERROR: Read1 and Read2 are not consistent, reached the end of the one before the other one\n";
+            errOut << "EXITING because of FATAL ERROR: read files are not consistent, reached the end of the one before the other one\n";
             errOut << "SOLUTION: Check you your input files: they may be corrupted\n";
             exitWithError(errOut.str(),std::cerr, P.inOut->logMain, EXIT_CODE_INPUT_FILES, P);
-        } else if (readStatus[0]==-1) {//finished with the stream
-            return -1;
         };
+    };
 
-        //combine two reads together
+    if (readStatus[0]==-1) {//finished with the stream
+        return -1;
+    };    
+    
+    if (P.outFilterBySJoutStage != 2) {
+        for (uint32 im=0; im<P.readNmates; im++) {//not readNends: the barcode quality will be calculated separately
+            for (uint64 ix=clipMates[im][0].clippedN; ix<readLengthOriginal[im]-clipMates[im][1].clippedN; ix++) {
+                qualHist[im][(uint8)Qual0[im][ix]]++;
+            };
+        };
+    };
+    
+    if (P.readNmates==2) {//combine two mates together
         Lread=readLength[0]+readLength[1]+1;
         readLengthPairOriginal=readLengthOriginal[0]+readLengthOriginal[1]+1;
         if (Lread>DEF_readSeqLengthMax) {
@@ -33,12 +43,13 @@ int ReadAlign::oneRead() {//process one read: load, map, write
             exitWithError(errOut.str(),std::cerr, P.inOut->logMain, EXIT_CODE_INPUT_FILES, P);
         };
 
-        Read1[0][readLength[0]]=MARK_FRAG_SPACER_BASE; //marker for spacer base
-        Qual1[0][readLength[0]]=0;
-        complementSeqNumbers(Read1[0]+readLength[0]+1,Read1[0]+readLength[0]+1,readLength[1]); //returns complement of Reads[ii]
+        //marker for spacer base
+        Read1[0][readLength[0]]=MARK_FRAG_SPACER_BASE;
+        
+        //copy 2nd mate into Read1[0] & reverse-complement
+        complementSeqNumbers(Read1[1],Read1[0]+readLength[0]+1,readLength[1]);//complement. Here Read1[1] is still the 2nd mate's numeric-sequence. Later Read1[1] will be reverse complement of the combined read.
         for (uint ii=0;ii<readLength[1]/2;ii++) {
-            swap(Read1[0][Lread-ii-1],Read1[0][ii+readLength[0]+1]); //reverse complement
-            swap(Qual1[0][Lread-ii-1],Qual1[0][ii+readLength[0]+1]); //reverse complement   ??? was Qualof the second mate populated
+            swap(Read1[0][Lread-ii-1],Read1[0][ii+readLength[0]+1]); //reverse
         };
 
     } else {//1 mate
@@ -52,13 +63,12 @@ int ReadAlign::oneRead() {//process one read: load, map, write
         readLength[1]=0;
 
     };
-
+      
     readFileType=readStatus[0];
 
     complementSeqNumbers(Read1[0],Read1[1],Lread); //returns complement of Reads[ii]
     for (uint ii=0;ii<Lread;ii++) {//reverse
         Read1[2][Lread-ii-1]=Read1[1][ii];
-        Qual1[1][Lread-ii-1]=Qual1[0][ii];
     };
 
     statsRA.readN++;
@@ -68,11 +78,19 @@ int ReadAlign::oneRead() {//process one read: load, map, write
     outFilterMismatchNmaxTotal=min(P.outFilterMismatchNmax, (uint) (P.outFilterMismatchNoverReadLmax*(readLength[0]+readLength[1])));
 
     //map the read
-    mapOneRead();
+    if (P.pGe.gType==101) {//SpliceGraph
+        mapOneReadSpliceGraph();
+    } else {//all other cases - standard alignment algorithm
+        mapOneRead();
+    };
 
     peOverlapMergeMap();
+    
     multMapSelect();
-    mappedFilter();
+    
+    mappedFilter();  
+    
+    transformGenome();//for now genome transformation happens after multimapper selection, and mapping filter
 
     if (!peOv.yes) {//if the alignment was not mates merged - otherwise the chimeric detection was already done
         chimericDetection();
@@ -95,3 +113,5 @@ int ReadAlign::oneRead() {//process one read: load, map, write
     return 0;
 
 };
+
+
