@@ -4,6 +4,13 @@
 
 void ReadAlign::outputAlignments() {
   
+    outBAMbytes=0;
+
+    if (P.pSolo.readStats.yes)
+        soloRead->readFlagReset();
+        
+    readAnnot.reset();
+    
     if (mapGen.pGe.gType==101) {//temporary
         ReadAlign::spliceGraphWriteSAM();
         return;
@@ -13,7 +20,7 @@ void ReadAlign::outputAlignments() {
     
     if (outFilterBySJoutPass) {//otherwise align is held for the 2nd stage of outFilterBySJout
         ////////////////////////////////////
-        if (unmapType<0) {//passed mappedFilter. Unmapped reads can have nTr>0 - "too many loci"
+        if (unmapType<0) {//passed mappedFilter. Unmapped reads can have nTr>0
             if (nTr>1) {//multimappers
                 statsRA.mappedReadsM++;
                 unmapType = -2; //not sure if this used
@@ -34,6 +41,11 @@ void ReadAlign::outputAlignments() {
         //the operations below are both for mapped and unmapped reads
         soloRead->readBar->getCBandUMI(Read0, Qual0, readLengthOriginal, readNameExtra[0], readFilesIndex, readName);
 
+        //transcripts: need to be run after CB/UMI are obtained to output CR/UR tags
+        if ( P.quant.trSAM.yes && unmapType<0) {//Aligned.toTranscriptome output, only for mapped
+            quantTranscriptome(chunkTr, nTr, trMult,  alignTrAll);
+        };        
+        
         soloRead->record((unmapType<0 ? nTr : 0), trMult, iReadAll, readAnnot); //need to supply nTr=0 for unmapped reads
 
         if (!P.pGe.transform.outSAM) {
@@ -53,6 +65,9 @@ void ReadAlign::outputAlignments() {
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 void ReadAlign::recordSJ(uint64 nTrO, Transcript **trO, OutSJ *cSJ)
 {//junction output for mapped reads (i.e. passed BySJout filtering)
+    if (!P.outSJ.yes)
+        return; //no SJ output
+    
     if ( P.outSJfilterReads=="All" || nTrO==1 ) {
         uint64 sjReadStartN=cSJ->N;
         for (uint64 iTr=0; iTr<nTrO; iTr++) {//write all transcripts junctions
@@ -140,16 +155,16 @@ void ReadAlign::writeSAM(uint64 nTrOutSAM, Transcript **trOutSAM, Transcript *tr
         
         ////////////////////////////////////////////////////////////////////////////////////////////////////////////
         ////// write SAM/BAM 
-        nTrOutSAM=min(P.outSAMmultNmax,nTrOutSAM); //number of aligns to write to SAM/BAM files            
+        auto nTrOutWrite=min(P.outSAMmultNmax,nTrOutSAM); //number of aligns to write to SAM/BAM files            
         
-        for (uint iTr=0;iTr<nTrOutSAM;iTr++) {//write all transcripts
+        for (uint iTr=0;iTr<nTrOutWrite;iTr++) {//write transcripts
             //mateMapped1 = true if a mate is present in this transcript
             bool mateMapped1[2]={false,false};
             mateMapped1[trOutSAM[iTr]->exons[0][EX_iFrag]]=true;
             mateMapped1[trOutSAM[iTr]->exons[trOutSAM[iTr]->nExons-1][EX_iFrag]]=true;
 
             if (P.outSAMbool) {//SAM output
-                outBAMbytes+=outputTranscriptSAM(*(trOutSAM[iTr]), nTr, iTr, (uint) -1, (uint) -1, 0, -1, NULL, outSAMstream);
+                outBAMbytes+=outputTranscriptSAM(*(trOutSAM[iTr]), nTrOutSAM, iTr, (uint) -1, (uint) -1, 0, -1, NULL, outSAMstream);
                 if (P.outSAMunmapped.keepPairs && P.readNmates>1 && ( !mateMapped1[0] || !mateMapped1[1] ) ) {//keep pairs && paired reads && one of the mates not mapped in this transcript //not readNends: this is alignment
                     outBAMbytes+= outputTranscriptSAM(*(trOutSAM[iTr]), 0, 0, (uint) -1, (uint) -1, 0, 4, mateMapped1, outSAMstream);
                 };
@@ -160,12 +175,12 @@ void ReadAlign::writeSAM(uint64 nTrOutSAM, Transcript **trOutSAM, Transcript *tr
 
                 if (P.outBAMunsorted) {//unsorted
                     for (uint imate=0; imate<P.readNmates; imate++) {//output each mate //not readNends: this is alignment
-                        outBAMunsorted->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], (imate>0 || iTr>0) ? 0 : (outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1])*2*nTrOutSAM);
+                        outBAMunsorted->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], (imate>0 || iTr>0) ? 0 : (outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1])*2*nTrOutWrite);
                     };
                     if (P.outSAMunmapped.keepPairs && P.readNmates>1 && ( !mateMapped1[0] || !mateMapped1[1] ) ) {//keep pairs && paired reads && one of the mates not mapped in this transcript //not readNends: this is alignment
                         alignBAM(*trOutSAM[iTr], 0, 0, mapGen.chrStart[trOutSAM[iTr]->Chr], (uint) -1, (uint) -1, 0, 4, mateMapped1, P.outSAMattrOrder, outBAMoneAlign, outBAMoneAlignNbytes);
                         for (uint imate=0; imate<P.readNmates; imate++) {//output each mate //not readNends: this is alignment
-                            outBAMunsorted->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], (imate>0 || iTr>0) ? 0 : (outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1])*2*nTrOutSAM);
+                            outBAMunsorted->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], (imate>0 || iTr>0) ? 0 : (outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1])*2*nTrOutWrite);
                         };
                     };
                 };
@@ -279,14 +294,18 @@ void ReadAlign::alignedAnnotation()
     };
     //solo-GeneFull
     if ( P.quant.geneFull.yes ) {
-        chunkTr->geneFullAlignOverlap(nTr, trMult, P.pSolo.strand, readAnnot);
-    };
+        chunkTr->geneFullAlignOverlap(nTr, trMult, P.pSolo.strand, readAnnot.annotFeatures[SoloFeatureTypes::GeneFull]);
+    };   
     //solo-Gene
     if ( P.quant.gene.yes ) {
         chunkTr->classifyAlign(trMult, nTr, readAnnot);
-    };            
-    //transcripts
-    if ( P.quant.trSAM.yes ) {
-        quantTranscriptome(chunkTr, nTr, trMult,  alignTrAll);
     };
+    //solo-GeneFull_ExonOverIntron
+    if ( P.quant.geneFull_ExonOverIntron.yes ) {
+        chunkTr->geneFullAlignOverlap_ExonOverIntron(nTr, trMult, P.pSolo.strand, readAnnot.annotFeatures[SoloFeatureTypes::GeneFull_ExonOverIntron], readAnnot.annotFeatures[SoloFeatureTypes::Gene]);
+    };
+    //solo-GeneFull_Ex50pAS
+    if ( P.quant.geneFull_Ex50pAS.yes ) {
+        chunkTr->alignExonOverlap(nTr, trMult, P.pSolo.strand, readAnnot.annotFeatures[SoloFeatureTypes::GeneFull_Ex50pAS]);
+    };    
 };
\ No newline at end of file
