@@ -9,14 +9,12 @@ uint ReadAlign::quantTranscriptome (Transcriptome *Tr, uint nAlignG, Transcript
     for (uint iag=0; iag<nAlignG; iag++) {//transform all alignments
 
         Transcript *align1=alignG[iag];
-        
-        align1->alignGenes={};
-        
+
         if (!P.quant.trSAM.indel && (align1->nDel>0 || align1->nIns>0) ) {
             //prevent indels if requested
             continue;
         };
-        if (!P.quant.trSAM.singleEnd && (P.readNmates==2 && align1->exons[0][EX_iFrag]==align1->exons[align1->nExons-1][EX_iFrag]) ) {
+        if (!P.quant.trSAM.singleEnd && (P.readNmates==2 && align1->exons[0][EX_iFrag]==align1->exons[align1->nExons-1][EX_iFrag]) ) {//not readNends: this is alignment
         //prevent single end alignments
             continue;
         };
@@ -72,18 +70,18 @@ uint ReadAlign::quantTranscriptome (Transcriptome *Tr, uint nAlignG, Transcript
 
         for (uint iatr=0;iatr<nAlignT;iatr++) {//write all transcripts
             alignBAM(alignT[iatr], nAlignT, iatr, 0, (uint) -1, (uint) -1, 0, -1, NULL, P.outSAMattrOrderQuant, outBAMoneAlign, outBAMoneAlignNbytes);
-            for (uint imate=0; imate<P.readNmates; imate++) {//output each mate
+            for (uint imate=0; imate<P.readNmates; imate++) {//output each mate //not readNends: this is alignment
                 outBAMquant->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], (imate>0 || iatr>0) ? 0 : (outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1])*2*nAlignT);
             };
         };
     };
 
-    //not used anymore, at Colin Dewey's request
+    //not used anymore per Colin Dewey's request
     //     if (nAlignT==0 && P.outSAMunmapped=="Within") {//read could be mapped to genome, but not transcriptome - output as unmapped
     //         uint unmapType=5;
     //         bool mateMapped[2]={false,false};
     //         alignBAM(*alignG[0], 0, 0, mapGen.chrStart[alignG[0]->Chr], (uint) -1, (uint) -1, 0,  unmapType, mateMapped, P.outSAMattrOrder);
-    //             for (uint imate=0; imate<P.readNmates; imate++) {//output each mate
+    //             for (uint imate=0; imate<P.readNmates; imate++) {//output each mate //not readNends: this is alignment
     //                 outBAMquant->unsortedOneAlign(outBAMoneAlign[imate], outBAMoneAlignNbytes[imate], imate>0 ? 0 : outBAMoneAlignNbytes[0]+outBAMoneAlignNbytes[1]);
     //             };
     //
