@@ -13,6 +13,8 @@ SoloFeature::SoloFeature(Parameters &Pin, ReadAlignChunk **RAchunk, Transcriptom
     switch (featureType) {
         case SoloFeatureTypes::Gene :
         case SoloFeatureTypes::GeneFull :
+        case SoloFeatureTypes::GeneFull_Ex50pAS :
+        case SoloFeatureTypes::GeneFull_ExonOverIntron :
         case SoloFeatureTypes::Velocyto :
             featuresNumber=Trans.nGe;
             break;
