@@ -138,8 +138,8 @@ void SoloFeature::countVelocyto()
             continue;
         
         nGenePerCB[iCB]+=geneC.size();
-        readFeatSum->stats.V[readFeatSum->stats.nUMIs] += nUMIperCB[iCB];
-        ++readFeatSum->stats.V[readFeatSum->stats.nCellBarcodes];
+        readFeatSum->stats.V[readFeatSum->stats.yesUMIs] += nUMIperCB[iCB];
+        ++readFeatSum->stats.V[readFeatSum->stats.yesCellBarcodes];
         
         if (countCellGeneUMI.size() < countCellGeneUMIindex[iCB+1] + geneC.size()*countMatStride) //allocated vector too small
             countCellGeneUMI.resize(countCellGeneUMI.size()*2);        
@@ -151,7 +151,10 @@ void SoloFeature::countVelocyto()
             countCellGeneUMIindex[iCB+1] += countMatStride;
         };
     };
-
+    
+    //for now only unique mappers are considered for velocyto
+    nReadPerCBtotal = nReadPerCB;
+    nReadPerCBunique = nReadPerCB;
     
     time(&rawTime);
     P.inOut->logMain << timeMonthDayTime(rawTime) << " ... Velocyto counting: finished collapsing UMIs" <<endl;
