@@ -77,12 +77,11 @@ void SoloReadBarcode::matchCBtoWL(string &cbSeq1, string &cbQual1, vector<uint64
         };
     };
     if (cbMatch1==0) {//no matches
-        //stats.V[stats.nNoMatch]++;
+        //stats.V[stats.noNoWLmatch]++;
         cbMatch1=-1;
     } else if (cbMatch1==1) {//1 match, no need to record the quality
         cbMatchString1 = to_string(cbMatchInd1[0]);
     } else if (!pSolo.CBmatchWL.mm1_multi) {//>1 matches, but this is not allowed
-        //stats.V[stats.nTooMany]++;
         cbMatch1=-3;
         cbMatchInd1.clear();
         cbMatchString1="";
@@ -93,35 +92,38 @@ void SoloReadBarcode::addStats(const int32 cbMatch1)
 {
     if (!pSolo.cbWLyes) //no stats if no WL
         return;
-    
-    if (cbMatch1>1) {
-        stats.V[stats.nMismatchToMultWL]++;
-        return;
-    };
-    
+       
     switch (cbMatch1) {
-        case 0:
+        case 0://exact matches
             cbReadCountExact[cbMatchInd[0]]++;//note that this simply counts reads per exact CB, no checks of genes or UMIs
-            stats.V[stats.nExactMatch]++;
+            stats.V[stats.yesWLmatchExact]++;
             break;
-        case 1:
-            stats.V[stats.nMismatchOneWL]++;
+        case 1: //one WL match counted here, but they may still get rejected in SoloReadFeature_inputRecords.cpp
+            stats.V[stats.yesOneWLmatchWithMM]++;
             break;
+        default: //multiple WL matches are counted here, but they may still get rejected in SoloReadFeature_inputRecords.cpp
+            stats.V[stats.yesMultWLmatchWithMM]++;
+            break;            
         case -1 :
-            stats.V[stats.nNoMatch]++;
+            stats.V[stats.noNoWLmatch]++;
             break;
         case -2 :
-            stats.V[stats.nNinCB]++;
+            stats.V[stats.noNinCB]++;
             break;
         case -3 :
-            stats.V[stats.nTooMany]++;
+            stats.V[stats.noTooManyWLmatches]++;
             break;
         case -11 :            
-            stats.V[stats.nNoCB]++;
+            stats.V[stats.noNoCB]++;//CB sequence cannot be extracted
             break;
         case -12 :
-            stats.V[stats.nMismatchesInMultCB]++;            
+            stats.V[stats.noTooManyMM]++;            
             break;
+        case -23:
+            stats.V[stats.noNinUMI]++;//UMIs are not allowed to have Ns
+            break;
+        case -24:
+            stats.V[stats.noUMIhomopolymer]++;            
     };
 };
 
@@ -129,12 +131,10 @@ void SoloReadBarcode::addStats(const int32 cbMatch1)
 bool SoloReadBarcode::convertCheckUMI()
 {//check UMIs, return if bad UMIs
     if (convertNuclStrToInt64(umiSeq,umiB)!=-1) {//convert and check for Ns
-        stats.V[stats.nNinUMI]++;//UMIs are not allowed to have Ns
         umiCheck=-23;
         return false;
     };
     if (umiB==homoPolymer[0] || umiB==homoPolymer[1] || umiB==homoPolymer[2] || umiB==homoPolymer[3]) {
-        stats.V[stats.nUMIhomopolymer]++;
         umiCheck=-24;        
         return false;
     };
@@ -269,6 +269,7 @@ void SoloReadBarcode::getCBandUMI(char **readSeq, char **readQual, uint64 *readL
             cbMatch=umiCheck;
             cbMatchString="";
             cbMatchInd.clear();
+            addStats(cbMatch);
             return;
         };
 
@@ -282,9 +283,13 @@ void SoloReadBarcode::getCBandUMI(char **readSeq, char **readQual, uint64 *readL
         matchCBtoWL(cbSeq, cbQual, pSolo.cbWL, cbMatch, cbMatchInd, cbMatchString);
 
         if ( cbMatch==0 || cbMatch==1 ) {
-        	cbSeqCorrected=pSolo.cbWLstr[cbMatchInd[0]];
+            if (pSolo.cbWLyes) {
+        	    cbSeqCorrected = pSolo.cbWLstr[cbMatchInd[0]];
+            } else {
+                cbSeqCorrected = cbSeq; //no WL - no correction
+            };
         } else {
-        	cbSeqCorrected="";
+        	cbSeqCorrected="-";
         };
         
     ///////////////////////////CB_UMI_Complex
@@ -298,14 +303,14 @@ void SoloReadBarcode::getCBandUMI(char **readSeq, char **readQual, uint64 *readL
         uint32 adapterStart=0;
         if (pSolo.adapterYes) {
             if (localAlignHammingDist(bSeq, pSolo.adapterSeq, adapterStart) > pSolo.adapterMismatchesNmax) {
-                stats.V[stats.nNoAdapter]++;
+                stats.V[stats.noNoAdapter]++;
                 cbMatch=-21;
                 return; //no adapter found
             };
         };
 
         if (!pSolo.umiV.extractBarcode(bSeq, bQual, adapterStart, umiSeq, umiQual)) {
-            stats.V[stats.nNoUMI]++;
+            stats.V[stats.noNoUMI]++;
             cbMatch=-22;
             return;
         };
@@ -334,18 +339,44 @@ void SoloReadBarcode::getCBandUMI(char **readSeq, char **readQual, uint64 *readL
             if (!cbMatchGood)
                 continue; //continue - to be able to record full cbSeq, cbQual, but no need to match to the WL
             
-            int32 cbMatch1;
-            vector<uint64> cbMatchInd1;
-            matchCBtoWL(cbSeq1, cbQual1, cb.wl[cbSeq1.size()], cbMatch1, cbMatchInd1, cbMatchString); //cbMatchString is not used for now, multiple matches are not allowed
-            if (cbMatch1<0) {//no match
-                cbMatchGood=false;
-                cbMatch = cbMatch1;
-            } else if (cbMatch1>0 && cbMatch>0) {//this barcode has >1 1MM match, or previous barcode had a mismatch
-                cbMatchGood=false;
-                cbMatch = -12; //marks mismatches in multiple barcodes
-            } else {//good match
-                cbMatchInd[0] += cb.wlFactor*(cbMatchInd1[0]+cb.wlAdd[cbSeq1.size()]);
-                cbMatch=max(cbMatch,cbMatch1);//1 wins over 0
+            uint64 cbLen1 = cbSeq1.size();
+            if ( pSolo.CBmatchWL.EditDist_2 ) {
+                cbMatch=0; //0: no MM, 1: >=1MM. 2: multi-match: not allowed for this option
+                uintCB cbB1;
+                int64 posN=convertNuclStrToInt64(cbSeq1,cbB1);
+                if (posN!=-1) {//Ns in barcode: no good match
+                    cbMatch = -2;
+                    cbMatchGood = false;
+                } else {
+                    int64 cbI=binarySearchExact<uint64>(cbB1,cb.wl[cbSeq1.size()].data(),cb.wl[cbLen1].size());
+                    if (cbI>=0) {//exact match
+                        cbMatchInd[0] += cb.wlFactor*(cbI+cb.wlAdd[cbLen1]);
+                    } else {//no exact match
+                        cbI=binarySearchExact<uint64>(cbB1,cb.wlEd[cbLen1].data(),cb.wlEd[cbLen1].size());
+                        if (cbI>=0) {//find match in the edited list
+                            cbMatch = 1; //>=1MM
+                            cbI = cb.wlEdInd[cbLen1][cbI];
+                            cbMatchInd[0] += cb.wlFactor*(cbI+cb.wlAdd[cbLen1]);
+                        } else {
+                            cbMatch = -1;
+                            cbMatchGood = false;
+                        };
+                    };
+                };
+            } else {// Exact or 1MM
+                int32 cbMatch1;
+                vector<uint64> cbMatchInd1;
+                matchCBtoWL(cbSeq1, cbQual1, cb.wl[cbLen1], cbMatch1, cbMatchInd1, cbMatchString); //cbMatchString is not used for now, multiple matches are not allowed
+                if (cbMatch1<0) {//no match
+                    cbMatchGood=false;
+                    cbMatch = cbMatch1;
+                } else if (cbMatch1>0 && cbMatch>0) {//this barcode has >1 1MM match, or previous barcode had a mismatch
+                    cbMatchGood=false;
+                    cbMatch = -12; //marks mismatches in multiple barcodes
+                } else {//good match
+                    cbMatchInd[0] += cb.wlFactor*(cbMatchInd1[0]+cb.wlAdd[cbLen1]);
+                    cbMatch=max(cbMatch,cbMatch1);//1 wins over 0
+                };
             };
         };
         cbSeq.pop_back();//remove last "_" from file
