@@ -18,15 +18,17 @@ public:
     uint32 homoPolymer[4];//homopolymer constants
 
     vector<uint32> cbReadCount;
-    map <uint32,uint32> cbReadCountMap;
+    map <uintCB,uint32> cbReadCountMap;
     
     vector<uint32> transcriptDistCount;
     
-    bool readInfoYes;
+    bool readInfoYes ,readIndexYes;
 
     fstream *streamReads;
 
     string cbSeq, umiSeq, cbQual, umiQual;
+
+    SoloReadFlagClass readFlag;
     
     SoloReadFeatureStats stats;
 
@@ -35,7 +37,8 @@ public:
     void addCounts(const SoloReadFeature &soloCBin);
     void addStats(const SoloReadFeature &soloCBin);
     void statsOut(ofstream &streamOut);
-    void inputRecords(uint32 **cbP, uint32 cbPstride, vector<uint32> &cbReadCountTotal, vector<readInfoStruct> &readInfo);
+    void inputRecords(uint32 **cbP, uint32 cbPstride, vector<uint32> &cbReadCountTotal, vector<readInfoStruct> &readInfo, SoloReadFlagClass &readFlagCounts,
+                      vector<uint32> &nReadPerCBunique1, vector<uint32> &nReadPerCBmulti1);
 
 private:
     const int32 featureType;
