@@ -25,7 +25,7 @@ Transcriptome::Transcriptome (Parameters &Pin) : P(Pin){
     };
     geStream.close();
 
-    if ( P.quant.trSAM.yes || P.quant.gene.yes ) {//load exon-transcript structures
+    if ( P.quant.trSAM.yes || P.quant.gene.yes || P.quant.geneFull_Ex50pAS.yes ) {//load exon-transcript structures
         //load tr and ex info
         ifstream & trinfo = ifstrOpen(trInfoDir+"/transcriptInfo.tab", ERROR_OUT, "SOLUTION: utilize --sjdbGTFfile /path/to/annotantions.gtf option at the genome generation step or mapping step",P);
         trinfo >> nTr;
@@ -93,7 +93,7 @@ Transcriptome::Transcriptome (Parameters &Pin) : P(Pin){
         };
     };
 
-    if ( P.quant.geneFull.yes ) {
+    if ( P.quant.geneFull.yes || P.quant.geneFull_ExonOverIntron.yes ) {
         ifstream & exinfo = ifstrOpen(trInfoDir+"/exonGeTrInfo.tab", ERROR_OUT, "SOLUTION: utilize --sjdbGTFfile /path/to/annotantions.gtf option at the genome generation step or mapping step", P);
         exinfo >> exG.nEx;
 
@@ -153,7 +153,7 @@ void Transcriptome::quantsOutput() {
     ofstream qOut(P.quant.geCount.outFile);
     qOut << "N_unmapped";
     for (int itype=0; itype<quants->geneCounts.nType; itype++) {
-        qOut << "\t" <<g_statsAll.unmappedAll;
+        qOut << "\t" <<g_statsAll.unmappedMismatch + g_statsAll.unmappedShort + g_statsAll.unmappedOther + g_statsAll.unmappedMulti;
     };
     qOut << "\n";
 
