@@ -117,11 +117,9 @@ void genomeSAindex(char * G, PackedArray & SA, Parameters & P, PackedArray & SAi
 void genomeSAindexChunk(char * G, PackedArray & SA, Parameters & P, PackedArray & SAi, uint iSA1, uint iSA2, Genome &mapGen)
 {
     uint* ind0=new uint[mapGen.pGe.gSAindexNbases];
-    uint* ind0a=new uint[mapGen.pGe.gSAindexNbases];
 
     for (uint ii=0; ii<mapGen.pGe.gSAindexNbases; ii++) {
         ind0[ii]=-1;//this is needed in case "AAA...AAA",i.e. indPref=0 is not present in the genome for some lengths
-        ind0a[ii]=-1;//this is needed in case "AAA...AAA",i.e. indPref=0 is not present in the genome for some lengths
     };
 
     PackedArray SAi1;
@@ -135,69 +133,20 @@ void genomeSAindexChunk(char * G, PackedArray & SA, Parameters & P, PackedArray
     int iL4;
     uint indFull=funCalcSAiFromSA(G,SA,mapGen,isa,mapGen.pGe.gSAindexNbases,iL4);
     while (isa<=iSA2) {//for all suffixes
-
-        /* testing
-        uint SAstr=SA[isa];
-        bool dirG = (SAstr>>mapGen.GstrandBit) == 0; //forward or reverse strand of the genome
-        SAstr &= mapGen.GstrandMask;
-        if (!dirG) SAstr=mapGen.nGenome-1-SAstr;
-        uint indPref1=0;
-        */
-
         for (uint iL=0; iL < mapGen.pGe.gSAindexNbases; iL++) {//calculate index
-            /*{//testing: old way
-                indPref1 <<= 2;
-
-                uint g1= (uint) G[dirG ? SAstr+iL : SAstr-iL]; //reverese if (-) strand
-
-                if (g1>3) {//if N, this suffix does not belong in SAi
-                    for (uint iL1=iL; iL1 < mapGen.pGe.gSAindexNbases; iL1++) {
-                        SAi1.writePacked(mapGen.genomeSAindexStart[iL1]+ind0[iL1],SAi[mapGen.genomeSAindexStart[iL1]+ind0[iL1]] | mapGen.SAiMarkNmaskC);
-                    };
-                } else //relying on the true code to break iL cycle
-                {
-                    if (!dirG) g1=3-g1; //complement if (-) strand
-
-                    indPref1 += (uint) g1;
-
-                    if ( indPref1 > ind0a[iL] || isa==0 ) {//new && good index, record it
-                        SAi1.writePacked(mapGen.genomeSAindexStart[iL]+indPref1, isa);
-                        for (uint ii=ind0a[iL]+1; ii<indPref1; ii++) {//index is not present, record to the last present suffix
-                            SAi1.writePacked(mapGen.genomeSAindexStart[iL]+ii, isa | mapGen.SAiMarkAbsentMaskC);
-                        };
-                        ind0a[iL]=indPref1;
-                    } else if ( indPref1 < ind0a[iL] ) {
-                        ostringstream errOut;
-                        errOut << "BUG: next index is smaller than previous, EXITING\n" <<flush;
-                        exitWithError(errOut.str(),std::cerr, P.inOut->logMain, EXIT_CODE_INPUT_FILES, P);
-                    };
-                };
-            };
-            */
 
             uint indPref = indFull >> (2*(mapGen.pGe.gSAindexNbases-1-iL));
-//             if (indPref!=indPref1)
-//                 cout<< iL <<" "<< isa <<" "<< indPref <<" "<<indPref1<<endl;
-
 
             if ( (int)iL==iL4 ) {//this suffix contains N and does not belong in SAi
                 for (uint iL1=iL; iL1 < mapGen.pGe.gSAindexNbases; iL1++) {
                     SAi.writePacked(mapGen.genomeSAindexStart[iL1]+ind0[iL1],SAi[mapGen.genomeSAindexStart[iL1]+ind0[iL1]] | mapGen.SAiMarkNmaskC);
-//                     if (SAi[mapGen.genomeSAindexStart[iL]+ind0[iL1]] != SAi1[mapGen.genomeSAindexStart[iL]+ind0[iL1]])
-//                         cout<< iL <<" "<< isa <<" "<< indPref <<" "<<indPref1<<endl;
-
                 };
                 break;//break the iL cycle
             };
 
             if ( indPref > ind0[iL] || isa==0 ) {//new && good index, record it
-                //testing
-//                 if (funCalcSAiFromSA(G,SA,isa,iL+1,P)!=indPref)
-//                     cout<< iL <<" "<< isa <<" "<< indPref <<" "<<funCalcSAiFromSA(G,SA,isa,iL+1,P)<<endl;
 
                 SAi.writePacked(mapGen.genomeSAindexStart[iL]+indPref, isa);
-//                 if (SAi[mapGen.genomeSAindexStart[iL]+indPref] != SAi1[mapGen.genomeSAindexStart[iL]+indPref])
-//                     cout<< iL <<" "<< isa <<" "<< indPref <<" "<<indPref1<<endl;
 
                 for (uint ii=ind0[iL]+1; ii<indPref; ii++) {//index is not present, record to the last present suffix
                     SAi.writePacked(mapGen.genomeSAindexStart[iL]+ii, isa | mapGen.SAiMarkAbsentMaskC);
@@ -209,15 +158,19 @@ void genomeSAindexChunk(char * G, PackedArray & SA, Parameters & P, PackedArray
                 errOut << "BUG: next index is smaller than previous, EXITING\n" <<flush;
                 exitWithError(errOut.str(),std::cerr, P.inOut->logMain, EXIT_CODE_INPUT_FILES, P);
             };
-
-
         };
 
         //find next index not equal to the current one
         funSAiFindNextIndex(G, SA, isaStep, isa, indFull, iL4, mapGen);//indFull and iL4 have been already defined at the previous step
-//         isa++;
-//         indFull=funCalcSAiFromSA(G,SA,isa,mapGen.pGe.gSAindexNbases,P,iL4);
+
     };//isa cycle
+
+    for (uint iL=0; iL < mapGen.pGe.gSAindexNbases; iL++) {//fill up unfilled indexes
+    	for (uint ii=mapGen.genomeSAindexStart[iL]+ind0[iL]+1; ii<mapGen.genomeSAindexStart[iL+1]; ii++) {
+    		SAi.writePacked(ii, mapGen.nSA | mapGen.SAiMarkAbsentMaskC);
+    	};
+    };
+
     delete [] ind0;
 
  };
