@@ -3,7 +3,8 @@
 
 #include "Parameters.h"
 #include "IncludeDefine.h"
+#include "Genome.h"
 
-uint genomeScanFastaFiles (Parameters *P, char* G, bool flagRun);
+uint genomeScanFastaFiles (Parameters &P, char* G, bool flagRun, Genome &mapGen);
 
 #endif
