@@ -2,6 +2,6 @@
 #define CODE_mapThreadsSpawn
 #include "Parameters.h"
 #include "ReadAlignChunk.h"
-void mapThreadsSpawn (Parameters *P, ReadAlignChunk** RAchunk);
+void mapThreadsSpawn (Parameters &P, ReadAlignChunk** RAchunk);
 
 #endif
\ No newline at end of file
